import sys
import os
sys.path.append(os.path.dirname(__file__))
import common as com
import numpy as np
import folium
from folium.plugins import HeatMap, MarkerCluster, Fullscreen, MousePosition



colors = {'darkOrange': '#FF8C00', 'cyan': '#0A8A9F'}


def addLayers(m):
    tiles = 'https://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}'
    folium.TileLayer(tiles=tiles, attr=(' '), name='Blue Marble').add_to(m)
    folium.TileLayer(tiles='cartoDBdark_matter', name='Black Diamond').add_to(m)
    return m


def addMousePosition(m):
    formatter = "function(num) {return L.Util.formatNum(num, 3) + ' º ';};"
    MousePosition(
        position='bottomright',
        separator=' | ',
        empty_string='NaN',
        lng_first=True,
        num_digits=20,
        prefix='Coordinates:',
        lat_formatter=formatter,
        lng_formatter=formatter
    ).add_to(m)
    return m


def addMarkers(m, df, variable, unit):
    normalized = com.normalize(df[variable])
    mc = MarkerCluster(name=variable+unit, options={'spiderfyOnMaxZoom':'False', 'disableClusteringAtZoom' : '4'})
    for i in range(len(df)):
        folium.CircleMarker(location=[df.lat[i], df.lon[i]], radius=(normalized[i] * 10), tooltip='%s: %f%s <br> date: %s' % (variable, df[variable][i], unit, df['time'][i]), color=colors['darkOrange'], fill=True).add_to(mc)
    mc.add_to(m)
    return m


def addFullScreen(m):
    Fullscreen(
        position='topright',
        title='Full Screen',
        title_cancel='Exit',
        force_separate_button=True
    ).add_to(m)
    return m





def heatMap(df, table, variable, unit):
    df.dropna(subset=[variable], inplace=True)
    df.reset_index(drop=True, inplace=True)
    normalized = com.normalize(df[variable])
    data = list(zip(df.lat, df.lon, normalized))

    m = folium.Map([df.lat.mean(), df.lon.mean()], tiles=None, zoom_start=3, control_scale=True, prefer_canvas=True)
    m.get_root().title = 'Map: ' + variable + unit
    m = addLayers(m)
    HeatMap(data, name='Data Density (%s)' % variable).add_to(m)
    m = addMarkers(m, df, variable, unit)
    m = addMousePosition(m)
    folium.LayerControl(collapsed=True).add_to(m)
    # m = addFullScreen(m)

    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)
    fname = dirPath + 'heatMap.html'
    if os.path.exists(fname):
        os.remove(fname)
    m.save(fname)
    com.openHTML(fname)
    return



import sys
import os
sys.path.append(os.path.dirname(__file__))
import numpy as np
import pandas as pd
from math import pi
import db
import plotHeat
from bokeh.plotting import figure, show, output_file
from bokeh.models import ColumnDataSource, HoverTool, DatetimeTickFormatter
from bokeh.layouts import column
import jupyterInline as jup
if jup.jupytered():
    from tqdm import tqdm_notebook as tqdm
else:
    from tqdm import tqdm


def timePanel(df, table, variable, unit, subject):
    TOOLS = "crosshair,pan,zoom_in,wheel_zoom,zoom_out,box_zoom,reset,save,"
    plotWidth, plotHeight = 800, 400
    markerSize = 9
    backgroundFillColor = None #'dimgray'
    fillColor = 'mediumslateblue'
    hoverFillColor = 'firebrick'
    hoverLineColor = 'white'
    lineColor = None
    fillAlpha = 0.5
    hoverAlpha = 0.8
    margin = 0.1    

    p2 = None
    if 'time' in df.columns:
        if isinstance(df['time'][0], str):
            df['time'] =  pd.to_datetime(df['time'], format='%Y-%m-%d')
        p2 = figure(tools=TOOLS, toolbar_location="right", title=subject, plot_width=plotWidth, plot_height=plotHeight, background_fill_color=backgroundFillColor)
        p2.yaxis.axis_label = variable + unit
        p2.xaxis.axis_label = 'Time'
        p2.xaxis.formatter = DatetimeTickFormatter(
                hours=["%Y-%m-%d"],
                days=["%Y-%m-%d"],
                months=["%Y-%m-%d"],
                years=["%Y-%m-%d"],
            )
        p2.xaxis.major_label_orientation = pi/4    
        source = ColumnDataSource({'x': df['time'], 'y': df[variable], 'Time': df['time']})      
        p2.circle('x', 'y', source = source, fill_color=fillColor, hover_fill_color=hoverFillColor, fill_alpha=fillAlpha, hover_alpha=hoverAlpha, line_color=lineColor, hover_line_color=hoverLineColor, size=markerSize)
        p2.add_tools(HoverTool(
            tooltips=[
                ('time', '@Time'),
                (variable+unit, '@y'),
            ],
            mode='mouse'
        ))
    return p2


def latPanel(df, table, variable, unit, subject):
    TOOLS = "crosshair,pan,zoom_in,wheel_zoom,zoom_out,box_zoom,reset,save,"
    plotWidth, plotHeight = 800, 400
    markerSize = 9
    backgroundFillColor = None #'dimgray'
    fillColor = 'mediumslateblue'
    hoverFillColor = 'firebrick'
    hoverLineColor = 'white'
    lineColor = None
    fillAlpha = 0.5
    hoverAlpha = 0.8
    margin = 0.1    

    p2 = None
    p2 = figure(tools=TOOLS, toolbar_location="right", title=subject, plot_width=plotWidth, plot_height=plotHeight, background_fill_color=backgroundFillColor)
    p2.yaxis.axis_label = variable + unit
    p2.xaxis.axis_label = 'Latitude'    
    p2.circle(df.lat, df[variable], fill_color=fillColor, hover_fill_color=hoverFillColor, fill_alpha=fillAlpha, hover_alpha=hoverAlpha, line_color=lineColor, hover_line_color=hoverLineColor, size=markerSize)
    p2.add_tools(HoverTool(
        tooltips=[
            ('latitude', '@x'),
            (variable+unit, '@y'),
        ],
        mode='mouse'
    ))
    return p2


def lonPanel(df, table, variable, unit, subject):
    TOOLS = "crosshair,pan,zoom_in,wheel_zoom,zoom_out,box_zoom,reset,save,"
    plotWidth, plotHeight = 800, 400
    markerSize = 9
    backgroundFillColor = None #'dimgray'
    fillColor = 'mediumslateblue'
    hoverFillColor = 'firebrick'
    hoverLineColor = 'white'
    lineColor = None
    fillAlpha = 0.5
    hoverAlpha = 0.8
    margin = 0.1    

    p2 = None
    p2 = figure(tools=TOOLS, toolbar_location="right", title=subject, plot_width=plotWidth, plot_height=plotHeight, background_fill_color=backgroundFillColor)
    p2.yaxis.axis_label = variable + unit
    p2.xaxis.axis_label = 'Longitude'
    p2.circle(df.lon, df[variable], fill_color=fillColor, hover_fill_color=hoverFillColor, fill_alpha=fillAlpha, hover_alpha=hoverAlpha, line_color=lineColor, hover_line_color=hoverLineColor, size=markerSize)
    p2.add_tools(HoverTool(
        tooltips=[
            ('longitude', '@x'),
            (variable+unit, '@y'),
        ],
        mode='mouse'
    ))
    return p2



def depthPanel(df, table, variable, unit, subject):
    TOOLS = "crosshair,pan,zoom_in,wheel_zoom,zoom_out,box_zoom,reset,save,"
    plotWidth, plotHeight = 800, 400
    markerSize = 9
    backgroundFillColor = None #'dimgray'
    fillColor = 'mediumslateblue'
    hoverFillColor = 'firebrick'
    hoverLineColor = 'white'
    lineColor = None
    fillAlpha = 0.5
    hoverAlpha = 0.8
    margin = 0.1    

    p2 = None
    if 'depth' in df.columns:
        p2 = figure(tools=TOOLS, toolbar_location="right", title=subject, plot_width=plotWidth, plot_height=plotHeight, background_fill_color=backgroundFillColor)
        p2.yaxis.axis_label = variable + unit
        p2.xaxis.axis_label = 'Depth [m]'
        p2.circle(df.depth, df[variable], fill_color=fillColor, hover_fill_color=hoverFillColor, fill_alpha=fillAlpha, hover_alpha=hoverAlpha, line_color=lineColor, hover_line_color=hoverLineColor, size=markerSize)
        p2.add_tools(HoverTool(
            tooltips=[
                ('depth', '@x'),
                (variable+unit, '@y'),
            ],
            mode='mouse'
        ))
    return p2



def dashboardPanels(df, table, variable):
    p=[]    
    time_sub = ''
    if 'depth' in df.columns:
        times = df['time'].sort_values().unique()  
        time_sub = 'time: ' + str(times[0]) + ' -- ' + str(times[-1])
    depth_sub = ''
    if 'depth' in df.columns:
        depth_sub = ', depth: %2.2f -- %2.2f' % (np.min(df.depth), np.max(df.depth)) + ' [m]'
    unit =  ' [' + db.getVar(table, variable).iloc[0]['Unit'] + ']'
    subject = time_sub + depth_sub
    unit =  ' [' + db.getVar(table, variable).iloc[0]['Unit'] + ']'

    plotHeat.heatMap(df, table, variable, unit)
    panel = timePanel(df, table, variable, unit, subject)
    if panel != None: p.append(panel)
    panel = latPanel(df, table, variable, unit, subject)    
    if panel != None: p.append(panel)        
    panel = lonPanel(df, table, variable, unit, subject)    
    if panel != None: p.append(panel)
    panel = depthPanel(df, table, variable, unit, subject)    
    if panel != None: p.append(panel)

    if len(p) > 0:
        if not jup.inline():          ## if jupyter is not the caller
            fname = 'dashboard'
            dirPath = 'embed/'
            if not os.path.exists(dirPath):
                os.makedirs(dirPath)        
            output_file(dirPath + fname + ".html", title=variable+unit)            
        show(column(p))
    return p





import sys
import os
sys.path.append(os.path.dirname(__file__))
import db
import validate



def spaceTime(tablesName, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2):
    if not validate.stringCheck(tablesName, field): return None
    args = [tablesName, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2]
    query = 'EXEC uspSpaceTime ?, ?, ?, ?, ?, ?, ?, ?, ?, ?'
    df = db.dbFetchStoredProc(query, args)   
    return df


def timeSeries(tablesName, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2):
    if not validate.stringCheck(tablesName, field): return None
    args = [tablesName, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2]
    query = 'EXEC uspTimeSeries ?, ?, ?, ?, ?, ?, ?, ?, ?, ?'
    df = db.dbFetchStoredProc(query, args)
    return df


def depthProfile(tablesName, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2):
    if not validate.stringCheck(tablesName, field): return None
    args = [tablesName, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2]
    query = 'EXEC uspDepthProfile ?, ?, ?, ?, ?, ?, ?, ?, ?, ?'
    df = db.dbFetchStoredProc(query, args)
    return df


def section(tablesName, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2):
    if not validate.stringCheck(tablesName, field): return None
    args = [tablesName, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2]
    query = 'EXEC uspSectionMap ?, ?, ?, ?, ?, ?, ?, ?, ?, ?'
    df = db.dbFetchStoredProc(query, args)
    return df



def spaceTimeList(tablesNames, fields, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2):
    df, dfs = None, []
    if not validate.listCheck(tablesNames, fields):
        return dfs
    for i in range(len(tablesNames)):    
        df = spaceTime(tablesNames[i], fields[i], dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2)
        dfs.append(df)
    return dfs


def timeSeriesList(tablesNames, fields, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2):
    df, dfs = None, []    
    if not validate.listCheck(tablesNames, fields):
        return dfs
    for i in range(len(tablesNames)):    
        df = timeSeries(tablesNames[i], fields[i], dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2)
        dfs.append(df)
    return dfs


def depthProfileList(tablesName, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2):
    df, dfs = None, []
    if not validate.listCheck(tablesNames, fields):
        return dfs
    for i in range(len(tablesNames)):    
        df = depthProfile(tablesNames[i], fields[i], dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2)
        dfs.append(df)
    return dfs


def sectionList(tablesName, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2):
    df, dfs = None, []
    if not validate.listCheck(tablesNames, fields):
        return dfs
    for i in range(len(tablesNames)):    
        df = section(tablesNames[i], fields[i], dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2)
        dfs.append(df)
    return dfs




from docopt import docopt
import sys
import os
sys.path.append(os.path.dirname(__file__))
import numpy as np
from scipy.interpolate import griddata
import pandas as pd
import db
import subset
from dashboard import dashboardPanels
import common as com
from datetime import datetime, timedelta
import time
from math import pi
from bokeh.plotting import figure, show, output_file
from bokeh.layouts import column
from bokeh.palettes import all_palettes
from bokeh.models import HoverTool, LinearColorMapper, BasicTicker, ColorBar, DatetimeTickFormatter
from bokeh.models.annotations import Title
from bokeh.embed import components
import jupyterInline as jup
if jup.jupytered():
    from tqdm import tqdm_notebook as tqdm
else:
    from tqdm import tqdm



def exportData(df, path):
    df.to_csv(path, index=False)    
    return


def structuredMap(df, table, variable, data, lats, lons, subs, frameTables, frameVars):
    times = df[df.columns[0]].unique()  
    lat = df.lat.unique()
    lon = df.lon.unique()
    shape = (len(lat), len(lon))
    
    depths, hours =  [None], [None]
    if 'depth' in df.columns:
        depths = df.depth.unique()
    if 'hour' in df.columns:
        hours = df.hour.unique()

    unit = com.getUnit(table, variable)
    for t in times:
        for h in hours:
            for z in depths:
                frame = df[df[df.columns[0]] == t]
                sub = variable + unit + ', ' + df.columns[0] + ': ' + str(t) 
                if h != None:
                    frame = frame[frame['hour'] == h]
                    sub = sub + ', hour: ' + str(h) + 'hr'
                if z != None:
                    frame = frame[frame['depth'] == z] 
                    sub = sub + ', depth: %2.2f' % z + ' [m]'  
                try:    
                    shot = frame[variable].values.reshape(shape)
                except Exception as e:
                    continue    
                lats.append(lat)
                lons.append(lon)
                data.append(shot)
                frameTables.append(table)
                frameVars.append(variable)
                subs.append(sub)
    return data, lats, lons, subs, frameTables, frameVars



def interpolatedMap(df, table, variable, data, lats, lons, subs, frameTables, frameVars):
    interpMethod = 'linear'
    times = df[df.columns[0]].sort_values().unique()  
    time_sub = df.columns[0] + ': ' + str(times[0]) + ' -- ' + str(times[-1])
    lat = np.array(df.lat)
    lon = np.array(df.lon)
    depths =  [None]
    depth_sub = ''
    if 'depth' in df.columns:
        depths = df.depth.sort_values().unique()    
        depth_sub = ', depth: %2.2f -- %2.2f' % (np.min(df.depth), np.max(df.depth)) + ' [m]'

    unit = com.getUnit(table, variable)
    sub = variable + unit + ', ' + time_sub
    sub = sub + depth_sub

    points = np.stack((lon, lat), axis=1)
    values = np.array(df[variable])

    # ################ uniform grid ################
    res_x, res_y = 0.1, 0.1    # in degrees
    lat_min, lat_max = np.min(df.lat) - res_y, np.max(df.lat) + res_y
    lon_min, lon_max = np.min(df.lon) - res_x, np.max(df.lon) + res_x
    x = np.arange(lon_min, lon_max, res_x)
    y = np.arange(lat_min, lat_max, res_y)

    # ############ grid on all data points #############
    # x = np.array(df.lon.sort_values().unique())
    # y = np.array(df.lat.sort_values().unique())

    grid_x, grid_y = np.meshgrid(x, y)
    interpData = griddata(points, values, (grid_x, grid_y), method=interpMethod)

    lats.append(lat)
    lons.append(lon)
    data.append(interpData)
    frameTables.append(table)
    frameVars.append(variable)
    subs.append(sub)
    return data, lats, lons, subs, frameTables, frameVars


def regionalMap(tables, variabels, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag):
    data, lats, lons, subs, frameTables, frameVars = [], [], [], [], [], []
    for i in tqdm(range(len(tables)), desc='overall'):
        df = subset.spaceTime(tables[i], variabels[i], dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2)        
        if len(df) < 1:
            com.printTQDM('%d: No matching entry found: Table: %s, Variable: %s ' % (i+1, tables[i], variabels[i]), err=True )
            continue
        com.printTQDM('%d: %s retrieved (%s).' % (i+1, variabels[i], tables[i]), err=False)

        ############### export retrieved data ###############
        if exportDataFlag:      
            dirPath = 'data/'
            if not os.path.exists(dirPath):
                os.makedirs(dirPath)     
            exportPath = dirPath + 'RM_' + tables[i] + '_' + variabels[i]               
            exportData(df, path=exportPath + '.csv')
        #####################################################

        if com.isGrid(tables[i], variabels[i]):
            data, lats, lons, subs, frameTables, frameVars = structuredMap(df, tables[i], variabels[i], data, lats, lons, subs, frameTables, frameVars)
        else:
            dashboardPanels(df, tables[i], variabels[i])
            # data, lats, lons, subs, frameTables, frameVars = interpolatedMap(df, tables[i], variabels[i], data, lats, lons, subs, frameTables, frameVars)
    
    bokehMap(data=data, subject=subs, fname=fname, lat=lats, lon=lons, tables=frameTables, variabels=frameVars)
    return




def bokehMap(data, subject, fname, lat, lon, tables, variabels):
    TOOLS="crosshair,pan,zoom_in,wheel_zoom,zoom_out,box_zoom,reset,save,"
    p = []
    for ind in range(len(data)):
        w, h = com.canvasRect(dw=np.max(lon[ind])-np.min(lon[ind]), dh=np.max(lat[ind])-np.min(lat[ind]))
        p1 = figure(tools=TOOLS, toolbar_location="right", title=subject[ind], plot_width=w, plot_height=h, x_range=(np.min(lon[ind]), np.max(lon[ind])), y_range=(np.min(lat[ind]), np.max(lat[ind])))
        p1.xaxis.axis_label = 'Longitude'
        p1.yaxis.axis_label = 'Latitude'
        unit = com.getUnit(tables[ind], variabels[ind])
        bounds = com.getBounds(variabels[ind])
        paletteName = com.getPalette(variabels[ind])
        low, high = bounds[0], bounds[1]
        if low == None:
            low, high = np.nanmin(data[ind].flatten()), np.nanmax(data[ind].flatten())
        color_mapper = LinearColorMapper(palette=paletteName, low=low, high=high)
        p1.image(image=[data[ind]], color_mapper=color_mapper, x=np.min(lon[ind]), y=np.min(lat[ind]), dw=np.max(lon[ind])-np.min(lon[ind]), dh=np.max(lat[ind])-np.min(lat[ind]))
        p1.add_tools(HoverTool(
            tooltips=[
                ('longitude', '$x'),
                ('latitude', '$y'),
                (variabels[ind]+unit, '@image'),
            ],
            mode='mouse'
        ))
        color_bar = ColorBar(color_mapper=color_mapper, ticker=BasicTicker(),
                        label_standoff=12, border_line_color=None, location=(0,0))
        p1.add_layout(color_bar, 'right')
        p.append(p1)

    if len(p) > 0:
        if not inline:      ## if jupyter is not the caller
            dirPath = 'embed/'
            if not os.path.exists(dirPath):
                os.makedirs(dirPath)        
            output_file(dirPath + fname + ".html", title="Regional Map")
        show(column(p))
    return




def main():
    tables = sys.argv[1]      
    variables = sys.argv[2]      
    dt1 = sys.argv[3]      
    dt2 = sys.argv[4]      
    lat1 = sys.argv[5]      
    lat2 = sys.argv[6]      
    lon1 = sys.argv[7]      
    lon2 = sys.argv[8]      
    depth1 = sys.argv[9]      
    depth2 = sys.argv[10]      
    fname = sys.argv[11]
    exportDataFlag = bool(int(sys.argv[12]))

    regionalMap(tables.split(','), variables.split(','), dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag)


inline = jup.inline()   # check if jupyter is calling this script
if __name__ == '__main__':
    main()
    




import platform
import sys
sys.dont_write_bytecode = True
import os
sys.path.append(os.path.dirname(__file__))
import common as com



###################  direct DB call  ####################
import pyodbc
import pandas.io.sql as sql
from pandas import DataFrame
def dbConnect(usr='ArmLab', psw='ArmLab2018', ip='128.208.239.15', port='1445', db='Opedia', TDS_Version='7.3'):
    try:
        server = ip + ',' + port
        if platform.system().lower().find('windows') != -1:
            conn = pyodbc.connect(DRIVER='{SQL Server}', SERVER='tcp:'+server, DATABASE=db, Uid=usr, Pwd=psw )
        elif platform.system().lower().find('darwin') != -1:
            conn = pyodbc.connect(DRIVER='/usr/local/lib/libtdsodbc.so', TDS_Version=TDS_Version, server=ip , port=port, DATABASE=db, Uid=usr, Pwd=psw )
        elif platform.system().lower().find('linux') != -1:
            conn = pyodbc.connect(DRIVER='/usr/lib/x86_64-linux-gnu/odbc/libtdsodbc.so', TDS_Version=TDS_Version, server=ip , port=port, DATABASE=db, uid=usr, pwd=psw)
    except Exception as e:
        com.halt('Database connection error. Error message: ' + str(e))
    return conn


def dbFetch(query):
    try:
        conn = dbConnect()
        df = sql.read_sql(query, conn)
        conn.close()
    except Exception as e:
        com.halt(str(e))
    return df


def dbFetchStoredProc(query, args):
    try:
        conn = dbConnect()
        cur = conn.cursor()
        if sys.version_info[0] >= 3:     # if python3 
            args = [ str(a) if a is not None else a for a in args ]
        cur.execute(query, args)
        df = cur.fetchall()
        cols = [column[0] for column in cur.description]
        df = DataFrame.from_records(df, columns=cols)
        conn.close()
    except Exception as e:
        com.halt(str(e))
    return df   
    





# ####################  Restful API call  ####################
# import requests
# import json
# from urllib.parse import urlencode
# import pandas as pd

# def arrangeColumns(df, tableName):
#     cols = list(df.columns)
#     timeCol = 'time'
#     if isClimatology(tableName):
#         timeCol = 'month'
#     if timeCol in cols:
#         oldIndex = cols.index(timeCol)
#         firstCol = cols[0]
#         cols[0] = timeCol
#         cols[oldIndex] = firstCol
#         df = df[cols]
#     return df

# def dbFetch(query):
#     payload = {'query': query}
#     url_safe_query = urlencode(payload)
#     url = com.getAPI_URL() + '/dataretrieval/query?' + url_safe_query
#     headers = {'Authorization': com.loadToken()}
#     response = requests.get(url, stream=True, headers=headers)
#     df = pd.DataFrame([json.loads(chunk) for chunk in response.iter_content(chunk_size=None)])
#     return df


# def dbFetchStoredProc(query, args):
#     payload = {
#     'tableName': args[0],    
#     'fields': args[1],
#     'dt1': args[2],
#     'dt2': args[3],
#     'lat1': args[4],
#     'lat2': args[5],
#     'lon1': args[6],
#     'lon2': args[7],
#     'depth1': args[8],
#     'depth2': args[9],
#     'spName': query.split(' ')[1]
#     }
#     url_encoded_params = urlencode(payload)
#     url = com.getAPI_URL() + '/dataretrieval/sp?' + url_encoded_params
#     headers = {'Authorization': com.loadToken()}
#     response = requests.get(url, stream=True, headers=headers)
#     df = pd.DataFrame([json.loads(chunk) for chunk in response.iter_content(chunk_size=None)])
#     df = arrangeColumns(df, args[0])
#     return df   




def getVar(tableName, varName):
    query = "SELECT * FROM tblVariables WHERE Table_Name='%s' AND Short_Name='%s'" % (tableName, varName)
    df = dbFetch(query)
    return df

def getTableName(varName):
    query = "SELECT * FROM tblVariables WHERE Table_Name='%s' AND Short_Name='%s'" % (tableName, varName)
    df = dbFetch(query)
    return df

def hasField(tableName, field):
    query = "SELECT COL_LENGTH('%s', '%s') AS RESULT " % (tableName, field)
    df = dbFetch(query)
    return False if df['RESULT'][0] == None else True

def isClimatology(tableName):
    return True if tableName.find('_Climatology') != -1 else False    




import sys
import os
sys.path.append(os.path.dirname(__file__))


def stringCheck(tablesName, field):
    valid = False
    if not isinstance(tablesName, str):
        print('Error: table name is expected to be string.')
        return valid
    if not isinstance(field, str):
        print('Error: variable name is expected to be string.')
        return valid
    valid = True
    return valid


def listCheck(tables, variables):
    valid = False
    if not isinstance(tables, list):
        print('Error: table names are expected to be a list of strings.')
        return valid
    if not isinstance(variables, list):
        print('Error: variable names are expected to be a list of strings.')
        return valid
    if len(tables) != len(variables):
        print('Error: please provide equal number of tables and variables.')
        return valid    
    valid = True
    return valid






import os
import sys
sys.path.append(os.path.dirname(__file__))
import common as com
import warnings


class API:
    """ construct and configure an instance of the RESTful API """
    def __init__(self, api_key):
        self.api_key = api_key
        self.tokenPath = com.getTokenPath()
        com.saveToken(self.tokenPath, self.api_key)




if sys.version_info < (3, 0):
    warnings.warn(
        ('Python 2 is not supported anymore. '
         'Please transition to Python 3 to be able to receive updates and fixes.'),
        UserWarning
    )

__version__ = '0.2.66'
__all__ = []
